import { Subject, Subscriber, Unsubscribable } from 'rxjs';
import { AdaptorTransmissionRole, TransmitterProfile } from '../interface/connector.interface';
import { MessageTransmissionBase } from './msg.transmission.base';
import { MessageReceiver as MessageReceiverInterface } from '../interface/transport.interface'
import { ConnectionAdapter } from '../connector/connector';

export class MessageReceiver extends MessageTransmissionBase implements MessageReceiverInterface<any> {
    receiverProfile: TransmitterProfile; // Property to hold the receiver profile
    transmissionRole: AdaptorTransmissionRole;
    incomingMessageBus: Subject<any> = new Subject()

    constructor(receiverProfile: TransmitterProfile) {
        super();
        this.transmissionRole = AdaptorTransmissionRole.Receiver;
        this.receiverProfile = receiverProfile;
        this.instantiateConnectionAdapter()
    }

    // Implement Subscribable's subscribe method. All messages pass through here
    subscribe(subscriber: Subscriber<any>): Unsubscribable {
        // Emit some value based on T (here for demonstration)
        subscriber.next({} as any);
        subscriber.complete();

        // Return an Unsubscribable object to allow unsubscription
        return {
            unsubscribe: () => {
                console.log('Unsubscribed');
            }
        };
    }

    private instantiateConnectionAdapter(): void {
        this.adapterService = new ConnectionAdapter()
        this.adapterService.subscribe((message: any) => {
            this.incomingMessageBus.next(message)
        })
    }
}
