
import { Observable, Observer, Subject, Unsubscribable } from 'rxjs';
import { TransmitterProfile, ReceiverProfile, AdaptorTransmissionRole } from '../interface/connector.interface';
import { MessageTransmissionBase as MessageTransmissionBaseInterface } from '../interface/transport.interface'
import { v4 as uuidv4 } from 'uuid'
import { MessageTransmitter } from './msg.transmitter';

export class MessageTransmissionBase implements MessageTransmissionBaseInterface<any> {
    transmitterProfile!: TransmitterProfile;
    receiverProfile!: ReceiverProfile;
    msgRepositoryService: any;
    transmissionRole!: AdaptorTransmissionRole;
    adaptorsArray!: any[];
    transmissionService: any;
    adapterService: any;
    incomingMessageBus!: Subject<any>;
    outgoingMessageBus!: Subject<any>;

    constructor() {
        // these are for self. Not to be confused with setting up credentials for connected clients in the context of websocket
        this.setTransmitter({
            id: uuidv4(),
            name: 'Transmitter' + process.env.PORT,
            dateCreated: new Date()
        })
        this.setReceiver({
            id: uuidv4(),
            name: 'Receiver',
            dateCreated: new Date()
        })
    }
    
    subscribe(observer: Partial<Observer<any>>): Unsubscribable {
        throw new Error('Method not implemented.');
    }

    // Pretty straight forward. Set itself straight away.
    setTransmitter(transmitterProfile: TransmitterProfile): MessageTransmitter {
        return new MessageTransmitter(transmitterProfile)
    }

    setReceiver(receiverProfile: ReceiverProfile): void {
        this.receiverProfile = receiverProfile
    }

    send(message: any): Observable<any> {
       throw new Error(`Method not implemented`)
    }
    emit(message: any): void {
        throw new Error('Method not implemented.');
    }
    emitStream(message: any): void {
        throw new Error('Method not implemented.');
    }
    subscribeMessages(messageFilter: any): Observable<any> {
        throw new Error('Method not implemented.');
    }

}