import { MessageTransmissionBase } from "./msg.transmission.base";
import { FisMessage, MessageTransmitter as MessageTransmitterInterface } from '../interface/transport.interface'
import { AdaptorTransmissionRole, TransmitterProfile } from "../interface/connector.interface";
import { filter, Observable, Observer, Subscription, takeWhile } from "rxjs";

export class MessageTransmitter extends MessageTransmissionBase implements MessageTransmitterInterface<any> {

    constructor(transmitterProfile: TransmitterProfile) {
        super()
        this.transmissionRole = AdaptorTransmissionRole.Transmitter
        this.transmitterProfile = transmitterProfile
    }

    send(message: FisMessage): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            this.transmissionService.send({
                transmitter: this.transmitterProfile,
                payload: message
            });
    
            const subscription: Subscription = this.incomingMessageBus
                .pipe(
                    // Filter for matching message IDs
                    filter(incomingMessage => incomingMessage.header.messageID === message.header.messageID),
                    // Complete the observable when the data is 'Complete'
                    takeWhile(incomingMessage => incomingMessage.data !== 'Complete')
                )
                .subscribe({
                    next: incomingMessage => {
                        response.next(incomingMessage);
                        // Complete the observable if data is 'Complete'
                        if (incomingMessage.data === 'Complete') {
                            response.complete();
                            subscription.unsubscribe();
                        }
                    }
                });
        });
    }

    emit(message: any): void {
        // emit base on receiver
    }

    emitStream(message: any): void {

    }
}